#include "fatal.hpp"
#include "display.hpp"

#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <errno.h>
#include <ncurses.h>

bool disp_inited=false;
int32 disp_cpx, disp_cpy;

void disp_init()
{
  if (disp_inited) fatal("already inited display");
  disp_inited=true;

  initscr();
  noecho();
  timeout(0);
  nonl();
  cbreak();
  keypad(stdscr,TRUE);
  notimeout(stdscr,FALSE);
  start_color();
  use_default_colors();

  init_pair(1,COLOR_BLACK,-1);
  init_pair(2,COLOR_RED,-1);
  init_pair(3,COLOR_GREEN,-1);
  init_pair(4,COLOR_YELLOW,-1);
  init_pair(5,COLOR_BLUE,-1);
  init_pair(6,COLOR_MAGENTA,-1);
  init_pair(7,COLOR_CYAN,-1);
  init_pair(8,COLOR_WHITE,-1);
  init_pair(9,COLOR_RED,COLOR_BLACK);
  init_pair(10,COLOR_GREEN,COLOR_BLACK);
  init_pair(11,COLOR_YELLOW,COLOR_BLACK);
  init_pair(12,COLOR_BLUE,COLOR_BLACK);
  init_pair(13,COLOR_MAGENTA,COLOR_BLACK);
  init_pair(14,COLOR_CYAN,COLOR_BLACK);
  init_pair(15,COLOR_WHITE,COLOR_BLACK);
  init_pair(16,COLOR_WHITE,COLOR_RED);
  init_pair(17,COLOR_BLACK,COLOR_GREEN);
  init_pair(18,COLOR_BLACK,COLOR_YELLOW);
  init_pair(19,COLOR_WHITE,COLOR_BLUE);
  init_pair(20,COLOR_BLACK,COLOR_MAGENTA);
  init_pair(21,COLOR_BLACK,COLOR_CYAN);
  init_pair(22,COLOR_BLACK,COLOR_WHITE);

  disp_curs(0,0);
  disp_putbox(0,0,COLS,LINES,' ',4);
  disp_update();
};

void disp_deinit()
{
  if (!disp_inited) return;
  disp_inited=false;

  endwin();
};

void disp_put(int32 x, int32 y, uint8 c, uint8 f)
{
  attrset(COLOR_PAIR(f));
  mvaddch(y,x,c);
};

void disp_curs(int32 x, int32 y)
{
  disp_cpx=x;
  disp_cpy=y;
};

void disp_putstring(int32 x, int32 y, const string &s, uint8 f)
{
  for (string::size_type i=0;i<s.size();++i)
    disp_put(x+i,y,s[i],f);
};

void disp_update()
{
  move(disp_cpy,disp_cpx);
  refresh();
};

void disp_putbox(int32 x, int32 y, int32 sx, int32 sy, uint8 c, uint8 f)
{
  for (int a=0;a<sx;++a)
    for (int b=0;b<sy;++b)
      disp_put(x+a,y+b,c,f);
};



static void intro ()
{
  if (disp_inited) disp_deinit();
  fprintf(stderr,"\n\n");
};

static void extro ()
{
  fprintf(stderr,"\n\n");
};

static void coredump ()
{
  fprintf(stderr,"I will now try to dump core\n");
  raise (SIGSEGV);
  fprintf(stderr,"I couldn't dump core. i will now kill myself\n");
  raise (SIGKILL);
};

static void fatalintro ()
{
  fprintf(stderr,
	  "FATAL ERROR ** FATAL ERROR ** FATAL ERROR\n"
	  "-----------------------------------------\n");
};

static void warningintro ()
{
  fprintf(stderr,"**  WARNING -- WARNING -- WARNING -- WARNING  **\n");
};

void fatal (const char *const msg)
{
  intro();
  fatalintro();
  fprintf(stderr,"%s\n", msg);
  coredump();
};

void fatal (const string &msg)
{
  fatal(msg.c_str());
};

void fatal_errno (const char *const msg)
{
  intro();
  fatalintro();
  fprintf(stderr,"%s\nerrno: %s\n", msg, strerror(errno));
  coredump();
};

void fatal_errno (const string &msg)
{
  fatal_errno(msg.c_str());
};

void warning (const char *const msg)
{
  intro();
  warningintro();
  fprintf(stderr,"%s\n", msg);
  extro();
};

void warning (const string &msg)
{
  warning(msg.c_str());
};
