/*

  yyyselector - a select() wrapper
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "selector.hpp"

#include "fatal.hpp"

#include <sys/time.h>
#include <errno.h>

using namespace yyyzeit;

selector::selector ()
{
  reset();
};

void selector::reset()
{
  FD_ZERO(&readfds);
  FD_ZERO(&writefds);
  infinity=true;
  timeout_expired=false;
};

void selector::add_read(const int handle)
{
  FD_SET(handle,&readfds);
};

void selector::add_write(const int handle)
{
  FD_SET(handle,&writefds);
};

void selector::add_exception(const int handle)
{
  FD_SET(handle,&exceptionfds);
};

void selector::settimeout(const zeit &timeout_)
{
  infinity=false;
  timeout=timeout_;
};

void selector::waitnow ()
{
  struct timeval *slarg;
  struct timeval slargrep;
 tryagain:
  if (infinity) {
    slarg=0;
  }else{
    zeit reltime(timeout-getcurrenttime());
    if (reltime.isnegative()) reltime=zerotime();
    reltime.convert_to_timeval(slargrep);
    slarg=&slargrep;
  };
  int retval = select (FD_SETSIZE,&readfds,&writefds,0,slarg);
  if (retval < 0) {
    if (errno==EINTR) goto tryagain;
    fatal_errno("selector::waitnow(): select returned error");
  };
  if (infinity) {
    timeout_expired = false;
  }else{
    timeout_expired = (getcurrenttime() >= timeout);
  };
};

const bool selector::isready_read(const int handle) const
{
  return FD_ISSET(handle,&readfds);
};

const bool selector::isready_write(const int handle) const
{
  return FD_ISSET(handle,&writefds);
};

const bool selector::isready_exception(const int handle) const
{
  return FD_ISSET(handle,&exceptionfds);
};
